/*
     Verify that this user/password is valid per a radius server database
     Return 1 if verified, 0 otherwise.
     
     Format of connection string 

     	<radius key server1>,<radius server1>,<radius key server2>,<radius server2>,
    	... ,<radius key server9>,<radius server9> 

     -------------------------------------------------------
     gremlin@portal-to-web.de		http://www.portal-to-web.de/tacacs
     

     Dependencies: You need to get the juniper networks libradius 
			(included in freebsd 4.x)

      License: tac_radius is free software; you can redistribute it
               and/or modify it under the terms of the BSD License
*/ 

/*#if defined(RADIUS) */
#include <stdio.h>
#include <string.h>

#include "tac_plus.h"
#include "radius.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <radlib.h>
#include "radlib_private.h"



int
radius_verify(user, users_passwd, str_conn)
char *user, *users_passwd;      /* Username and gived password   */
char *str_conn;                 /* String connection to database */
{
	struct rad_handle *rh;
	struct in_addr addr;
	int res;
	char *token, *cp;
	char *server[10];
	char *key[10];
	int i=0,j=0;
    	char l_err[200];
  	int err;


/*
	 open the radius handle 
*/
        if ((rh=rad_auth_open()) == NULL ) {
                report (LOG_ERR,"Can't open rad_open");
                return (1);
        } else {
/*                report (LOG_INFO,"rad_open successfull"); */
        }


/*
	split the line from the config file into pairs with radiusserver and 
	radius key
*/
	cp = strdup (str_conn);                				/* Make writable copy.  */
	while ((token = strsep (&cp,",")) != NULL ) {
		if ( (  i % 2 ) == 0 ) {				/* 0,2,4,... are the radius keys 	*/
			key[i]=(char *)malloc(sizeof (token) + sizeof(char));		
									/* get some mem 		 	*/
			sprintf (key[i],"%s\0",token);			/* add the value gotten into a array 	*/
		} else {
			server[i-1]=(char *)malloc(sizeof (token) + sizeof(char)); 	
									/* 1,3,5... are the radius hosts	*/
			sprintf (server[i-1],"%s\0",token);		/* malloc and add to array 		*/
		}
		i++;
	}


/*
	for each server and key pair gotten from the config file, do a
	rad_add_server 
*/
	for (; j < i; j=j+2) {
  		report(LOG_INFO, "In verify_radius : Before ldap_init : radiusserver = %s, radiuskey = %s ", 
				server[j],key[j]);
        	if ((rad_add_server(rh,server[j],0,key[j],2,2)) == 0) {
                /*	report (LOG_INFO,"rad_add_server successfull for %s",server[j]); */
        	} else {
                	report (LOG_ERR,"Error in rad_add_server for %s",server[j]);
                	return (1);
        	}
	}


/*
	create a radius request for ACCESS
*/


	if ((rad_create_request(rh ,RAD_ACCESS_REQUEST)) == 0) {
		/* report (LOG_INFO,"rad_create_request successfull"); */
	} else {
		report (LOG_ERR,"Error in rad_create_request");
		return (1);
	}


/*
	prepare the radius request

	1. make a in_addr from the requesting peer ip address
	2. put in username 
	3. put in password
	4. insert the in_addr into the rad_request
	5. set the RAD_CONNECT_INFO to "via tacacs+"

*/
	inet_aton(session.peer, &addr);  

	rad_put_string(rh,RAD_USER_NAME,user);
	rad_put_string(rh,RAD_USER_PASSWORD,users_passwd);

	rad_put_addr(rh,RAD_NAS_IP_ADDRESS,addr); 
	rad_put_string(rh,RAD_CONNECT_INFO,"via tacacs+");

/*
	some debug messages before sending the radius request

  	report(LOG_INFO, "In verify_radius : Before rad_send : user = %s, passwd = %s", user,"********");
  	report(LOG_INFO, "In verify_radius : Before rad_send : peer %s", session.peer);
*/

/*
	go forest, go!
	send the radius request and hope, that libradius does a good job
*/

	res=rad_send_request(rh);

/*
	is the user authenticated?
	if yes, return 0, else 1
*/
	if (res == RAD_ACCESS_ACCEPT ) {
		report  (LOG_INFO,"Request accepted\n");
		return (0);
	} else {
		report (LOG_INFO,"Request denied %i\n",res);
		return (1);
	}
}
	
/*#endif*/ /* RADIUS */
